@@ -12,11 +12,12 @@
 #include "config/touch.h"
 #include "config/tablet.h"
 #include "config/libinput.h"
-#include "resize_indicator.h"
+#include "resize-indicator.h"
 #include "theme.h"
 
 enum view_placement_policy {
-	LAB_PLACE_CENTER = 0,
+	LAB_PLACE_INVALID = 0,
+	LAB_PLACE_CENTER,
 	LAB_PLACE_CURSOR,
 	LAB_PLACE_AUTOMATIC
 };
@@ -54,6 +55,7 @@ struct rcxml {
 	bool allow_tearing;
 	bool reuse_output_mode;
 	enum view_placement_policy placement_policy;
+	bool xwayland_persistence;
 
 	/* focus */
 	bool focus_follow_mouse;
@@ -93,6 +95,7 @@ struct rcxml {
 
 	/* graphics tablet */
 	struct tablet_config {
+		bool force_mouse_emulation;
 		char *output_name;
 		struct wlr_fbox box;
 		enum rotation rotation;
@@ -116,6 +119,7 @@ struct rcxml {
 	enum tiling_events_mode snap_tiling_events_mode;
 
 	enum resize_indicator_mode resize_indicator;
+	bool resize_draw_contents;
 
 	struct {
 		int popuptime;
@@ -140,6 +144,13 @@ struct rcxml {
 
 	/* Menu */
 	unsigned int menu_ignore_button_release_period;
+
+	/* Magnifier */
+	int mag_width;
+	int mag_height;
+	float mag_scale;
+	float mag_increment;
+	bool mag_filter;
 };
 
 extern struct rcxml rc;
