@@ -3,22 +3,32 @@
 #define LABWC_TABLET_H
 
 #include <wayland-server-core.h>
+#include <wlr/types/wlr_tablet_v2.h>
 
 struct seat;
 struct wlr_device;
 struct wlr_input_device;
 
 struct drawing_tablet {
+	struct wlr_input_device *wlr_input_device;
 	struct seat *seat;
 	struct wlr_tablet *tablet;
+	struct wlr_tablet_v2_tablet *tablet_v2;
 	double x, y;
+	double distance;
+	double pressure;
+	double tilt_x, tilt_y;
+	double rotation;
+	double slider;
+	double wheel_delta;
 	struct {
+		struct wl_listener proximity;
 		struct wl_listener axis;
 		struct wl_listener tip;
 		struct wl_listener button;
 		struct wl_listener destroy;
-		// no interest in proximity events
 	} handlers;
+	struct wl_list link; /* seat.tablets */
 };
 
 void tablet_init(struct seat *seat, struct wlr_input_device *wlr_input_device);
