@@ -42,6 +42,7 @@
 #include <wlr/types/wlr_tearing_control_v1.h>
 #include <wlr/types/wlr_text_input_v3.h>
 #include <wlr/types/wlr_input_method_v2.h>
+#include <wlr/types/wlr_tablet_v2.h>
 #include <wlr/util/log.h>
 #include "config/keybind.h"
 #include "config/rcxml.h"
@@ -192,6 +193,10 @@ struct seat {
 	struct wl_listener touch_motion;
 	struct wl_listener touch_frame;
 
+	struct wl_list tablets;
+	struct wl_list tablet_tools;
+	struct wl_list tablet_pads;
+
 	struct wl_listener constraint_commit;
 	struct wl_listener pressed_surface_destroy;
 
@@ -205,6 +210,12 @@ struct seat {
 struct lab_data_buffer;
 struct workspace;
 
+enum lab_cycle_dir {
+	LAB_CYCLE_DIR_NONE,
+	LAB_CYCLE_DIR_FORWARD,
+	LAB_CYCLE_DIR_BACKWARD,
+};
+
 struct server {
 	struct wl_display *wl_display;
 	struct wl_event_loop *wl_event_loop;  /* Can be used for timer events */
@@ -313,7 +324,7 @@ struct server {
 	struct wlr_gamma_control_manager_v1 *gamma_control_manager_v1;
 	struct wl_listener gamma_control_set_gamma;
 
-	struct session_lock *session_lock;
+	struct session_lock_manager *session_lock_manager;
 
 	struct wlr_foreign_toplevel_manager_v1 *foreign_toplevel_manager;
 
@@ -333,6 +344,9 @@ struct server {
 	struct wlr_input_method_manager_v2 *input_method_manager;
 	struct wlr_text_input_manager_v3 *text_input_manager;
 
+	struct wlr_tablet_manager_v2 *tablet_manager;
+	struct wlr_security_context_manager_v1 *security_context_manager_v1;
+
 	/* Set when in cycle (alt-tab) mode */
 	struct osd_state {
 		struct view *cycle_view;
@@ -341,6 +355,8 @@ struct server {
 		struct wlr_scene_tree *preview_parent;
 		struct wlr_scene_node *preview_anchor;
 		struct multi_rect *preview_outline;
+		enum lab_cycle_dir initial_direction;
+		bool initial_keybind_contained_shift;
 	} osd_state;
 
 	struct theme *theme;
@@ -434,12 +450,6 @@ struct view *desktop_topmost_focusable_view(struct server *server);
  */
 void desktop_update_top_layer_visiblity(struct server *server);
 
-enum lab_cycle_dir {
-	LAB_CYCLE_DIR_NONE,
-	LAB_CYCLE_DIR_FORWARD,
-	LAB_CYCLE_DIR_BACKWARD,
-};
-
 /**
  * desktop_cycle_view - return view to 'cycle' to
  * @start_view: reference point for finding next view to cycle to
