@@ -15,6 +15,7 @@
 #include "common/string-helpers.h"
 #include "debug.h"
 #include "labwc.h"
+#include "magnifier.h"
 #include "menu/menu.h"
 #include "osd.h"
 #include "output-virtual.h"
@@ -23,6 +24,7 @@
 #include "ssd.h"
 #include "view.h"
 #include "workspaces.h"
+#include "input/keyboard.h"
 
 enum action_arg_type {
 	LAB_ACTION_ARG_STR = 0,
@@ -77,7 +79,9 @@ enum action_type {
 	ACTION_TYPE_SHOW_MENU,
 	ACTION_TYPE_TOGGLE_MAXIMIZE,
 	ACTION_TYPE_MAXIMIZE,
+	ACTION_TYPE_UNMAXIMIZE,
 	ACTION_TYPE_TOGGLE_FULLSCREEN,
+	ACTION_TYPE_SET_DECORATIONS,
 	ACTION_TYPE_TOGGLE_DECORATIONS,
 	ACTION_TYPE_TOGGLE_ALWAYS_ON_TOP,
 	ACTION_TYPE_TOGGLE_ALWAYS_ON_BOTTOM,
@@ -110,6 +114,10 @@ enum action_type {
 	ACTION_TYPE_SHADE,
 	ACTION_TYPE_UNSHADE,
 	ACTION_TYPE_TOGGLE_SHADE,
+	ACTION_TYPE_TOGGLE_TABLET_MOUSE_EMULATION,
+	ACTION_TYPE_TOGGLE_MAGNIFY,
+	ACTION_TYPE_ZOOM_IN,
+	ACTION_TYPE_ZOOM_OUT
 };
 
 const char *action_names[] = {
@@ -130,7 +138,9 @@ const char *action_names[] = {
 	"ShowMenu",
 	"ToggleMaximize",
 	"Maximize",
+	"UnMaximize",
 	"ToggleFullscreen",
+	"SetDecorations",
 	"ToggleDecorations",
 	"ToggleAlwaysOnTop",
 	"ToggleAlwaysOnBottom",
@@ -163,6 +173,10 @@ const char *action_names[] = {
 	"Shade",
 	"Unshade",
 	"ToggleShade",
+	"ToggleTabletMouseEmulation",
+	"ToggleMagnify",
+	"ZoomIn",
+	"ZoomOut",
 	NULL
 };
 
@@ -329,6 +343,7 @@ action_arg_from_xml_node(struct action *action, const char *nodename, const char
 		break;
 	case ACTION_TYPE_TOGGLE_MAXIMIZE:
 	case ACTION_TYPE_MAXIMIZE:
+	case ACTION_TYPE_UNMAXIMIZE:
 		if (!strcmp(argument, "direction")) {
 			enum view_axis axis = view_axis_parse(content);
 			if (axis == VIEW_AXIS_NONE) {
@@ -340,6 +355,17 @@ action_arg_from_xml_node(struct action *action, const char *nodename, const char
 			goto cleanup;
 		}
 		break;
+	case ACTION_TYPE_SET_DECORATIONS:
+		if (!strcmp(argument, "decorations")) {
+			enum ssd_mode mode = ssd_mode_parse(content);
+			action_arg_add_int(action, argument, mode);
+			goto cleanup;
+		}
+		if (!strcasecmp(argument, "forceSSD")) {
+			action_arg_add_bool(action, argument, parse_bool(content, false));
+			goto cleanup;
+		}
+		break;
 	case ACTION_TYPE_RESIZE_RELATIVE:
 		if (!strcmp(argument, "left") || !strcmp(argument, "right") ||
 				!strcmp(argument, "top") || !strcmp(argument, "bottom")) {
@@ -415,6 +441,19 @@ action_arg_from_xml_node(struct action *action, const char *nodename, const char
 			goto cleanup;
 		}
 		break;
+	case ACTION_TYPE_AUTO_PLACE:
+		if (!strcmp(argument, "policy")) {
+			enum view_placement_policy policy =
+				view_placement_parse(content);
+			if (policy == LAB_PLACE_INVALID) {
+				wlr_log(WLR_ERROR, "Invalid argument for action %s: '%s' (%s)",
+						action_names[action->type], argument, content);
+			} else {
+				action_arg_add_int(action, argument, policy);
+			}
+			goto cleanup;
+		}
+		break;
 	}
 
 	wlr_log(WLR_ERROR, "Invalid argument for action %s: '%s'",
@@ -662,6 +701,26 @@ run_if_action(struct view *view, struct server *server, struct action *action)
 	return !strcmp(branch, "then");
 }
 
+static bool
+shift_is_pressed(struct server *server)
+{
+	uint32_t modifiers = wlr_keyboard_get_modifiers(
+			&server->seat.keyboard_group->keyboard);
+	return modifiers & WLR_MODIFIER_SHIFT;
+}
+
+static void
+start_window_cycling(struct server *server, enum lab_cycle_dir direction)
+{
+	/* Remember direction so it can be followed by subsequent key presses */
+	server->osd_state.initial_direction = direction;
+	server->osd_state.initial_keybind_contained_shift =
+		shift_is_pressed(server);
+	server->osd_state.cycle_view = desktop_cycle_view(server,
+		server->osd_state.cycle_view, direction);
+	osd_update(server);
+}
+
 void
 actions_run(struct view *activator, struct server *server,
 	struct wl_list *actions, uint32_t resize_edges)
@@ -671,6 +730,9 @@ actions_run(struct view *activator, struct server *server,
 		return;
 	}
 
+	/* This cancels any pending on-release keybinds */
+	keyboard_reset_current_keybind();
+
 	struct view *view;
 	struct action *action;
 	wl_list_for_each(action, actions, link) {
@@ -749,14 +811,10 @@ actions_run(struct view *activator, struct server *server,
 			}
 			break;
 		case ACTION_TYPE_NEXT_WINDOW:
-			server->osd_state.cycle_view = desktop_cycle_view(server,
-				server->osd_state.cycle_view, LAB_CYCLE_DIR_FORWARD);
-			osd_update(server);
+			start_window_cycling(server, LAB_CYCLE_DIR_FORWARD);
 			break;
 		case ACTION_TYPE_PREVIOUS_WINDOW:
-			server->osd_state.cycle_view = desktop_cycle_view(server,
-				server->osd_state.cycle_view, LAB_CYCLE_DIR_BACKWARD);
-			osd_update(server);
+			start_window_cycling(server, LAB_CYCLE_DIR_BACKWARD);
 			break;
 		case ACTION_TYPE_RECONFIGURE:
 			kill(getpid(), SIGHUP);
@@ -781,11 +839,28 @@ actions_run(struct view *activator, struct server *server,
 					/*store_natural_geometry*/ true);
 			}
 			break;
+		case ACTION_TYPE_UNMAXIMIZE:
+			if (view) {
+				enum view_axis axis = action_get_int(action,
+					"direction", VIEW_AXIS_BOTH);
+				view_maximize(view, view->maximized & ~axis,
+					/*store_natural_geometry*/ true);
+			}
+			break;
 		case ACTION_TYPE_TOGGLE_FULLSCREEN:
 			if (view) {
 				view_toggle_fullscreen(view);
 			}
 			break;
+		case ACTION_TYPE_SET_DECORATIONS:
+			if (view) {
+				enum ssd_mode mode = action_get_int(action,
+					"decorations", LAB_SSD_MODE_FULL);
+				bool force_ssd = action_get_bool(action,
+					"forceSSD", false);
+				view_set_decorations(view, mode, force_ssd);
+			}
+			break;
 		case ACTION_TYPE_TOGGLE_DECORATIONS:
 			if (view) {
 				view_toggle_decorations(view);
@@ -884,6 +959,9 @@ actions_run(struct view *activator, struct server *server,
 			}
 			break;
 		case ACTION_TYPE_MOVETO_CURSOR:
+			wlr_log(WLR_ERROR,
+				"Action MoveToCursor is deprecated. To ensure your config works in future labwc "
+				"releases, please use <action name=\"AutoPlace\" policy=\"cursor\">");
 			if (view) {
 				view_move_to_cursor(view);
 			}
@@ -911,6 +989,11 @@ actions_run(struct view *activator, struct server *server,
 				if (action->type == ACTION_TYPE_SEND_TO_DESKTOP) {
 					view_move_to_workspace(view, target);
 					follow = action_get_bool(action, "follow", true);
+
+					/* Ensure that the focus is not on another desktop */
+					if (!follow && server->active_view == view) {
+						desktop_focus_topmost_view(server);
+					}
 				}
 				if (follow) {
 					workspaces_switch_to(target,
@@ -1018,10 +1101,10 @@ actions_run(struct view *activator, struct server *server,
 			break;
 		case ACTION_TYPE_AUTO_PLACE:
 			if (view) {
-				struct wlr_box geometry = view->pending;
-				if (placement_find_best(view, &geometry)) {
-					view_move(view, geometry.x, geometry.y);
-				}
+				enum view_placement_policy policy =
+					action_get_int(action, "policy", LAB_PLACE_AUTOMATIC);
+				view_place_by_policy(view,
+					/* allow_cursor */ true, policy);
 			}
 			break;
 		case ACTION_TYPE_TOGGLE_TEARING:
@@ -1046,6 +1129,18 @@ actions_run(struct view *activator, struct server *server,
 				view_set_shade(view, false);
 			}
 			break;
+		case ACTION_TYPE_TOGGLE_TABLET_MOUSE_EMULATION:
+			rc.tablet.force_mouse_emulation = !rc.tablet.force_mouse_emulation;
+			break;
+		case ACTION_TYPE_TOGGLE_MAGNIFY:
+			magnify_toggle(server);
+			break;
+		case ACTION_TYPE_ZOOM_IN:
+			magnify_set_scale(server, MAGNIFY_INCREASE);
+			break;
+		case ACTION_TYPE_ZOOM_OUT:
+			magnify_set_scale(server, MAGNIFY_DECREASE);
+			break;
 		case ACTION_TYPE_INVALID:
 			wlr_log(WLR_ERROR, "Not executing unknown action");
 			break;
