@@ -189,6 +189,12 @@ fill_window_rule(char *nodename, char *content)
 		current_window_rule->window_type = parse_window_type(content);
 	} else if (!strcasecmp(nodename, "matchOnce")) {
 		set_bool(content, &current_window_rule->match_once);
+	} else if (!strcasecmp(nodename, "sandboxEngine")) {
+		free(current_window_rule->sandbox_engine);
+		current_window_rule->sandbox_engine = xstrdup(content);
+	} else if (!strcasecmp(nodename, "sandboxAppId")) {
+		free(current_window_rule->sandbox_app_id);
+		current_window_rule->sandbox_app_id = xstrdup(content);
 
 	/* Event */
 	} else if (!strcmp(nodename, "event")) {
@@ -289,6 +295,11 @@ fill_region(char *nodename, char *content)
 static void
 fill_action_query(char *nodename, char *content, struct action *action)
 {
+	if (!action) {
+		wlr_log(WLR_ERROR, "No parent action for query: %s=%s", nodename, content);
+		return;
+	}
+
 	string_truncate_at_pattern(nodename, ".keybind.keyboard");
 	string_truncate_at_pattern(nodename, ".mousebind.context.mouse");
 
@@ -308,7 +319,7 @@ fill_action_query(char *nodename, char *content, struct action *action)
 			action_arg_add_querylist(action, "query");
 			queries = action_get_querylist(action, "query");
 		}
-		current_view_query = znew(*current_view_query);
+		current_view_query = view_query_create();
 		wl_list_append(queries, &current_view_query->link);
 	}
 
@@ -316,6 +327,12 @@ fill_action_query(char *nodename, char *content, struct action *action)
 		current_view_query->identifier = xstrdup(content);
 	} else if (!strcasecmp(nodename, "title")) {
 		current_view_query->title = xstrdup(content);
+	} else if (!strcmp(nodename, "type")) {
+		current_view_query->window_type = parse_window_type(content);
+	} else if (!strcasecmp(nodename, "sandboxEngine")) {
+		current_view_query->sandbox_engine = xstrdup(content);
+	} else if (!strcasecmp(nodename, "sandboxAppId")) {
+		current_view_query->sandbox_app_id = xstrdup(content);
 	}
 }
 
@@ -323,6 +340,11 @@ static void
 fill_child_action(char *nodename, char *content, struct action *parent,
 	const char *branch_name)
 {
+	if (!parent) {
+		wlr_log(WLR_ERROR, "No parent action for branch: %s=%s", nodename, content);
+		return;
+	}
+
 	string_truncate_at_pattern(nodename, ".keybind.keyboard");
 	string_truncate_at_pattern(nodename, ".mousebind.context.mouse");
 	string_truncate_at_pattern(nodename, ".then.action");
@@ -381,6 +403,8 @@ fill_keybind(char *nodename, char *content)
 	} else if (!current_keybind) {
 		wlr_log(WLR_ERROR, "expect <keybind key=\"\"> element first. "
 			"nodename: '%s' content: '%s'", nodename, content);
+	} else if (!strcasecmp(nodename, "onRelease")) {
+		set_bool(content, &current_keybind->on_release);
 	} else if (!strcasecmp(nodename, "layoutDependent")) {
 		set_bool(content, &current_keybind->use_syms_only);
 	} else if (!strcmp(nodename, "name.action")) {
@@ -871,13 +895,12 @@ entry(xmlNode *node, char *nodename, char *content)
 	} else if (!strcasecmp(nodename, "reuseOutputMode.core")) {
 		set_bool(content, &rc.reuse_output_mode);
 	} else if (!strcmp(nodename, "policy.placement")) {
-		if (!strcmp(content, "automatic")) {
-			rc.placement_policy = LAB_PLACE_AUTOMATIC;
-		} else if (!strcmp(content, "cursor")) {
-			rc.placement_policy = LAB_PLACE_CURSOR;
-		} else {
+		rc.placement_policy = view_placement_parse(content);
+		if (rc.placement_policy == LAB_PLACE_INVALID) {
 			rc.placement_policy = LAB_PLACE_CENTER;
 		}
+	} else if (!strcasecmp(nodename, "xwaylandPersistence.core")) {
+		set_bool(content, &rc.xwayland_persistence);
 	} else if (!strcmp(nodename, "name.theme")) {
 		rc.theme_name = xstrdup(content);
 	} else if (!strcmp(nodename, "cornerradius.theme")) {
@@ -1011,6 +1034,10 @@ entry(xmlNode *node, char *nodename, char *content)
 		} else {
 			wlr_log(WLR_ERROR, "Invalid value for <resize popupShow />");
 		}
+	} else if (!strcasecmp(nodename, "drawContents.resize")) {
+		set_bool(content, &rc.resize_draw_contents);
+	} else if (!strcasecmp(nodename, "mouseEmulation.tablet")) {
+		set_bool(content, &rc.tablet.force_mouse_emulation);
 	} else if (!strcasecmp(nodename, "mapToOutput.tablet")) {
 		rc.tablet.output_name = xstrdup(content);
 	} else if (!strcasecmp(nodename, "rotate.tablet")) {
@@ -1036,6 +1063,16 @@ entry(xmlNode *node, char *nodename, char *content)
 		}
 	} else if (!strcasecmp(nodename, "ignoreButtonReleasePeriod.menu")) {
 		rc.menu_ignore_button_release_period = atoi(content);
+	} else if (!strcasecmp(nodename, "width.magnifier")) {
+		rc.mag_width = atoi(content);
+	} else if (!strcasecmp(nodename, "height.magnifier")) {
+		rc.mag_height = atoi(content);
+	} else if (!strcasecmp(nodename, "initScale.magnifier")) {
+		set_float(content, &rc.mag_scale);
+	} else if (!strcasecmp(nodename, "increment.magnifier")) {
+		set_float(content, &rc.mag_increment);
+	} else if (!strcasecmp(nodename, "useFilter.magnifier")) {
+		set_bool(content, &rc.mag_filter);
 	}
 }
 
@@ -1210,6 +1247,7 @@ rcxml_init(void)
 	rc.doubleclick_time = 500;
 	rc.scroll_factor = 1.0;
 
+	rc.tablet.force_mouse_emulation = false;
 	rc.tablet.output_name = NULL;
 	rc.tablet.rotation = 0;
 	rc.tablet.box = (struct wlr_fbox){0};
@@ -1237,11 +1275,18 @@ rcxml_init(void)
 		| LAB_VIEW_CRITERIA_NO_SKIP_WINDOW_SWITCHER;
 
 	rc.resize_indicator = LAB_RESIZE_INDICATOR_NEVER;
+	rc.resize_draw_contents = true;
 
 	rc.workspace_config.popuptime = INT_MIN;
 	rc.workspace_config.min_nr_workspaces = 1;
 
 	rc.menu_ignore_button_release_period = 250;
+
+	rc.mag_width = 400;
+	rc.mag_height = 400;
+	rc.mag_scale = 2.0;
+	rc.mag_increment = 0.2;
+	rc.mag_filter = true;
 }
 
 static void
@@ -1468,6 +1513,10 @@ post_processing(void)
 		wlr_log(WLR_INFO, "load default window switcher fields");
 		load_default_window_switcher_fields();
 	}
+
+	if (rc.mag_scale <= 0.0) {
+		rc.mag_scale = 1.0;
+	}
 }
 
 static void
@@ -1476,6 +1525,8 @@ rule_destroy(struct window_rule *rule)
 	wl_list_remove(&rule->link);
 	zfree(rule->identifier);
 	zfree(rule->title);
+	zfree(rule->sandbox_engine);
+	zfree(rule->sandbox_app_id);
 	action_list_free(&rule->actions);
 	zfree(rule);
 }
@@ -1544,7 +1595,8 @@ validate(void)
 	/* Window-rule criteria */
 	struct window_rule *rule, *rule_tmp;
 	wl_list_for_each_safe(rule, rule_tmp, &rc.window_rules, link) {
-		if (!rule->identifier && !rule->title && rule->window_type < 0) {
+		if (!rule->identifier && !rule->title && rule->window_type < 0
+				&& !rule->sandbox_engine && !rule->sandbox_app_id) {
 			wlr_log(WLR_ERROR, "Deleting rule %p as it has no criteria", rule);
 			rule_destroy(rule);
 		}
