@@ -38,6 +38,15 @@ struct keyinfo {
 
 static bool should_cancel_cycling_on_next_key_release;
 
+static struct keybind *cur_keybind;
+
+/* Called on --reconfigure to prevent segfault when handling release keybinds */
+void
+keyboard_reset_current_keybind(void)
+{
+	cur_keybind = NULL;
+}
+
 static void
 change_vt(struct server *server, unsigned int vt)
 {
@@ -226,6 +235,20 @@ is_modifier_key(xkb_keysym_t sym)
 	}
 }
 
+static bool
+is_modifier(struct wlr_keyboard *wlr_keyboard, uint32_t evdev_keycode)
+{
+	const xkb_keysym_t *syms = NULL;
+	int nr_syms = xkb_state_key_get_syms(wlr_keyboard->xkb_state,
+		evdev_keycode + 8, &syms);
+	for (int i = 0; i < nr_syms; i++) {
+		if (is_modifier_key(syms[i])) {
+			return true;
+		}
+	}
+	return false;
+}
+
 static struct keyinfo
 get_keyinfo(struct wlr_keyboard *wlr_keyboard, uint32_t evdev_keycode)
 {
@@ -266,7 +289,7 @@ get_keyinfo(struct wlr_keyboard *wlr_keyboard, uint32_t evdev_keycode)
 	 * bool last_key_was_a_modifier = last_modifiers != modifiers;
 	 * last_modifiers = modifiers;
 	 * if (last_key_was_a_modifier) {
-	 *	key_state_remove_last_pressed_key(last_pressed_keycode);
+	 *   key_state_remove_last_pressed_key(last_pressed_keycode);
 	 * }
 	 */
 	keyinfo.modifiers = wlr_keyboard_get_modifiers(wlr_keyboard);
@@ -360,6 +383,16 @@ handle_menu_keys(struct server *server, struct keysyms *syms)
 	}
 }
 
+static void
+toggle_direction(enum lab_cycle_dir *direction)
+{
+	if (*direction == LAB_CYCLE_DIR_FORWARD) {
+		*direction = LAB_CYCLE_DIR_BACKWARD;
+	} else if (*direction == LAB_CYCLE_DIR_BACKWARD) {
+		*direction = LAB_CYCLE_DIR_FORWARD;
+	}
+}
+
 static void
 handle_cycle_view_key(struct server *server, struct keyinfo *keyinfo)
 {
@@ -374,21 +407,36 @@ handle_cycle_view_key(struct server *server, struct keyinfo *keyinfo)
 
 	/* cycle to next */
 	if (!keyinfo->is_modifier) {
-		bool back_key = false;
+		enum lab_cycle_dir direction = server->osd_state.initial_direction;
 		for (int i = 0; i < keyinfo->translated.nr_syms; i++) {
 			if (keyinfo->translated.syms[i] == XKB_KEY_Up
 					|| keyinfo->translated.syms[i] == XKB_KEY_Left) {
-				back_key = true;
-				break;
+				direction = LAB_CYCLE_DIR_BACKWARD;
+				goto miss_shift_toggle;
+			}
+			if (keyinfo->translated.syms[i] == XKB_KEY_Down
+					|| keyinfo->translated.syms[i] == XKB_KEY_Right) {
+				direction = LAB_CYCLE_DIR_FORWARD;
+				goto miss_shift_toggle;
 			}
 		}
-		bool backwards = (keyinfo->modifiers & WLR_MODIFIER_SHIFT) || back_key;
 
-		enum lab_cycle_dir dir = backwards
-			? LAB_CYCLE_DIR_BACKWARD
-			: LAB_CYCLE_DIR_FORWARD;
+		bool shift_is_pressed = keyinfo->modifiers & WLR_MODIFIER_SHIFT;
+		if (shift_is_pressed != server->osd_state.initial_keybind_contained_shift) {
+			/*
+			 * Shift reverses the direction - unless shift was part of the
+			 * original keybind in which case we do the opposite.
+			 * For example with S-A-Tab bound to PreviousWindow, shift with
+			 * subsequent key presses should carry on cycling backwards.
+			 */
+			toggle_direction(&direction);
+		}
+
+	/* Only one direction modifier is allowed, either arrow keys OR shift */
+miss_shift_toggle:
+
 		server->osd_state.cycle_view = desktop_cycle_view(server,
-			server->osd_state.cycle_view, dir);
+			server->osd_state.cycle_view, direction);
 		osd_update(server);
 	}
 }
@@ -407,7 +455,18 @@ handle_compositor_keybindings(struct keyboard *keyboard,
 		keyinfo.is_modifier);
 
 	if (event->state == WL_KEYBOARD_KEY_STATE_RELEASED) {
-		return handle_key_release(server, event->keycode);
+		if (cur_keybind && cur_keybind->on_release) {
+			key_state_bound_key_remove(event->keycode);
+			if (seat->server->session_lock_manager->locked
+					|| seat->active_client_while_inhibited) {
+				cur_keybind = NULL;
+				return true;
+			}
+			actions_run(NULL, server, &cur_keybind->actions, 0);
+			return true;
+		} else {
+			return handle_key_release(server, event->keycode);
+		}
 	}
 
 	/* Catch C-A-F1 to C-A-F12 to change tty */
@@ -424,7 +483,7 @@ handle_compositor_keybindings(struct keyboard *keyboard,
 	if (seat->active_client_while_inhibited) {
 		return false;
 	}
-	if (seat->server->session_lock) {
+	if (seat->server->session_lock_manager->locked) {
 		return false;
 	}
 
@@ -443,16 +502,17 @@ handle_compositor_keybindings(struct keyboard *keyboard,
 	/*
 	 * Handle compositor keybinds
 	 */
-	struct keybind *keybind =
-		match_keybinding(server, &keyinfo, keyboard->is_virtual);
-	if (keybind) {
+	cur_keybind = match_keybinding(server, &keyinfo, keyboard->is_virtual);
+	if (cur_keybind) {
 		/*
 		 * Update key-state before action_run() because the action
 		 * might lead to seat_focus() in which case we pass the
 		 * 'pressed-sent' keys to the new surface.
 		 */
 		key_state_store_pressed_key_as_bound(event->keycode);
-		actions_run(NULL, server, &keybind->actions, 0);
+		if (!cur_keybind->on_release) {
+			actions_run(NULL, server, &cur_keybind->actions, 0);
+		}
 		return true;
 	}
 
@@ -528,7 +588,13 @@ keyboard_key_notify(struct wl_listener *listener, void *data)
 	}
 
 	if (handled) {
-		if (event->state == WL_KEYBOARD_KEY_STATE_PRESSED) {
+		/*
+		 * We do not start the repeat-timer on pressed modifiers (like
+		 * Super_L) because it is only for our own internal use with
+		 * keybinds and it messes up modifier-onRelease-keybinds.
+		 */
+		if (!is_modifier(keyboard->wlr_keyboard, event->keycode)
+				&& event->state == WL_KEYBOARD_KEY_STATE_PRESSED) {
 			start_keybind_repeat(seat->server, keyboard, event);
 		}
 	} else if (!input_method_keyboard_grab_forward_key(keyboard, event)) {
