@@ -17,7 +17,7 @@
 #include "common/list.h"
 #include "common/mem.h"
 #include "common/nodename.h"
-#include "common/scaled_font_buffer.h"
+#include "common/scaled-font-buffer.h"
 #include "common/scene-helpers.h"
 #include "common/spawn.h"
 #include "common/string-helpers.h"
@@ -605,6 +605,11 @@ xml_tree_walk(xmlNode *node, struct server *server)
 			continue;
 		}
 		if (!strcasecmp((char *)n->name, "item")) {
+			if (!current_menu) {
+				wlr_log(WLR_ERROR,
+					"ignoring <item> without parent <menu>");
+				continue;
+			}
 			in_item = true;
 			traverse(n, server);
 			in_item = false;
@@ -666,7 +671,7 @@ parse_xml(const char *filename, struct server *server)
 		struct path *path = wl_container_of(elm, path, link);
 		FILE *stream = fopen(path->string, "r");
 		if (!stream) {
-			return;
+			continue;
 		}
 		wlr_log(WLR_INFO, "read menu file %s", path->string);
 		parse_stream(server, stream);
@@ -697,22 +702,23 @@ menu_get_full_width(struct menu *menu)
 	return width + max_child_width;
 }
 
+/**
+ * get_submenu_position() - get output layout coordinates of menu window
+ * @item: the menuitem that triggers the submenu (static or dynamic)
+ */
 static struct wlr_box
 get_submenu_position(struct menuitem *item, enum menu_align align)
 {
 	struct wlr_box pos = { 0 };
 	struct menu *menu = item->parent;
 	struct theme *theme = menu->server->theme;
-	int lx = menu->scene_tree->node.x;
-	int ly = menu->scene_tree->node.y;
+	pos.x = menu->scene_tree->node.x;
+	pos.y = menu->scene_tree->node.y;
 
 	if (align & LAB_MENU_OPEN_RIGHT) {
-		pos.x = lx + menu->size.width - theme->menu_overlap_x;
-	} else {
-		pos.x = lx;
+		pos.x += menu->size.width - theme->menu_overlap_x;
 	}
-	int rel_y = item->tree->node.y;
-	pos.y = ly + rel_y - theme->menu_overlap_y;
+	pos.y += item->tree->node.y - theme->menu_overlap_y;
 	return pos;
 }
 
@@ -845,7 +851,7 @@ init_windowmenu(struct server *server)
 		fill_item("name.action", "ToggleMaximize");
 		current_item = item_create(menu, _("Fullscreen"), false);
 		fill_item("name.action", "ToggleFullscreen");
-		current_item = item_create(menu, _("Roll up/down"), false);
+		current_item = item_create(menu, _("Roll Up/Down"), false);
 		fill_item("name.action", "ToggleShade");
 		current_item = item_create(menu, _("Decorations"), false);
 		fill_item("name.action", "ToggleDecorations");
@@ -854,14 +860,14 @@ init_windowmenu(struct server *server)
 
 		/* Workspace sub-menu */
 		struct menu *workspace_menu = menu_create(server, "workspaces", "");
-		current_item = item_create(workspace_menu, _("Move left"), false);
+		current_item = item_create(workspace_menu, _("Move Left"), false);
 		/*
 		 * <action name="SendToDesktop"><follow> is true by default so
 		 * GoToDesktop will be called as part of the action.
 		 */
 		fill_item("name.action", "SendToDesktop");
 		fill_item("to.action", "left");
-		current_item = item_create(workspace_menu, _("Move right"), false);
+		current_item = item_create(workspace_menu, _("Move Right"), false);
 		fill_item("name.action", "SendToDesktop");
 		fill_item("to.action", "right");
 		current_item = separator_create(workspace_menu, "");
@@ -932,6 +938,8 @@ menu_free(struct menu *menu)
 	 */
 	wlr_scene_node_destroy(&menu->scene_tree->node);
 	wl_list_remove(&menu->link);
+	zfree(menu->id);
+	zfree(menu->label);
 	zfree(menu);
 }
 
@@ -1112,19 +1120,9 @@ create_pipe_menu(struct pipe_context *ctx)
 	/* Set menu-widths before configuring */
 	post_processing(ctx->server);
 
-	/*
-	 * TODO:
-	 * (1) Combine this with the code in get_submenu_position()
-	 *     and/or menu_configure()
-	 * (2) Take into account menu_overlap_{x,y}
-	 */
 	enum menu_align align = ctx->item->parent->align;
-	int x = pipe_parent->scene_tree->node.x;
-	int y = pipe_parent->scene_tree->node.y + ctx->item->tree->node.y;
-	if (align & LAB_MENU_OPEN_RIGHT) {
-		x += pipe_parent->size.width;
-	}
-	menu_configure(pipe_menu, x, y, align);
+	struct wlr_box pos = get_submenu_position(ctx->item, align);
+	menu_configure(pipe_menu, pos.x, pos.y, align);
 
 	validate(ctx->server);
 
