@@ -11,7 +11,7 @@
 #include "common/mem.h"
 #include "input/ime.h"
 #include "input/tablet.h"
-#include "input/tablet_pad.h"
+#include "input/tablet-pad.h"
 #include "input/input.h"
 #include "input/keyboard.h"
 #include "input/key-state.h"
@@ -31,19 +31,6 @@ input_device_destroy(struct wl_listener *listener, void *data)
 		wl_list_remove(&keyboard->key.link);
 		wl_list_remove(&keyboard->modifier.link);
 		keyboard_cancel_keybind_repeat(keyboard);
-		/*
-		 * If the active keyboard on the seat is destroyed, fall back
-		 * to the keyboard from keyboard group so we can respond to
-		 * wl_seat.get_keyboard requests with wl_keyboard.keymap event.
-		 * This prevents Chromium from crashing when started just after
-		 * the active keyboard is destroyed.
-		 */
-		struct wlr_keyboard *active_keyboard =
-			wlr_seat_get_keyboard(input->seat->seat);
-		if (!active_keyboard || active_keyboard == keyboard->wlr_keyboard) {
-			wlr_seat_set_keyboard(input->seat->seat,
-				&input->seat->keyboard_group->keyboard);
-		}
 	}
 	free(input);
 }
@@ -410,6 +397,7 @@ seat_update_capabilities(struct seat *seat)
 			caps |= WL_SEAT_CAPABILITY_KEYBOARD;
 			break;
 		case WLR_INPUT_DEVICE_POINTER:
+		case WLR_INPUT_DEVICE_TABLET_TOOL:
 			caps |= WL_SEAT_CAPABILITY_POINTER;
 			break;
 		case WLR_INPUT_DEVICE_TOUCH:
@@ -516,6 +504,7 @@ focus_change_notify(struct wl_listener *listener, void *data)
 		}
 		if (view) {
 			view_set_activated(view, true);
+			tablet_pad_enter_surface(seat, surface);
 		}
 		server->active_view = view;
 	}
@@ -565,6 +554,10 @@ seat_init(struct server *server)
 	}
 	wlr_cursor_attach_output_layout(seat->cursor, server->output_layout);
 
+	wl_list_init(&seat->tablets);
+	wl_list_init(&seat->tablet_tools);
+	wl_list_init(&seat->tablet_pads);
+
 	input_handlers_init(seat);
 }
 
@@ -602,6 +595,7 @@ seat_reconfigure(struct server *server)
 	struct input *input;
 	cursor_reload(seat);
 	overlay_reconfigure(seat);
+	keyboard_reset_current_keybind();
 	wl_list_for_each(input, &seat->inputs, link) {
 		switch (input->wlr_input_device->type) {
 		case WLR_INPUT_DEVICE_KEYBOARD:
@@ -633,7 +627,7 @@ seat_focus(struct seat *seat, struct wlr_surface *surface, bool is_lock_surface)
 	 * lock screen may lose focus and become impossible to unlock.
 	 */
 	struct server *server = seat->server;
-	if (server->session_lock && !is_lock_surface) {
+	if (server->session_lock_manager->locked && !is_lock_surface) {
 		return;
 	}
 
@@ -648,6 +642,15 @@ seat_focus(struct seat *seat, struct wlr_surface *surface, bool is_lock_surface)
 		return;
 	}
 
+	if (!wlr_seat_get_keyboard(seat->seat)) {
+		/*
+		 * wlr_seat_keyboard_notify_enter() sends wl_keyboard.modifiers,
+		 * but it may crash some apps (e.g. Chromium) if
+		 * wl_keyboard.keymap is not sent beforehand.
+		 */
+		wlr_seat_set_keyboard(seat->seat, &seat->keyboard_group->keyboard);
+	}
+
 	/*
 	 * Key events associated with keybindings (both pressed and released)
 	 * are not sent to clients. When changing surface-focus it is therefore
