@@ -7,6 +7,7 @@
  */
 
 #include <assert.h>
+#include <strings.h>
 #include "common/mem.h"
 #include "common/scene-helpers.h"
 #include "labwc.h"
@@ -191,7 +192,7 @@ ssd_create(struct view *view, bool active)
 	ssd_titlebar_create(ssd);
 	if (view->ssd_titlebar_hidden) {
 		/* Ensure we keep the old state on Reconfigure or when exiting fullscreen */
-		ssd_titlebar_hide(ssd);
+		ssd_set_titlebar(ssd, false);
 	}
 	ssd->margin = ssd_thickness(view);
 	ssd_set_active(ssd, active);
@@ -245,10 +246,10 @@ ssd_update_geometry(struct ssd *ssd)
 			ssd_extents_update(ssd);
 			ssd->state.geometry = current;
 		}
-		bool maximized = (ssd->view->maximized == VIEW_AXIS_BOTH);
+		bool maximized = ssd->view->maximized == VIEW_AXIS_BOTH;
 		if (ssd->state.was_maximized != maximized) {
-			ssd_border_update(ssd);
 			ssd_titlebar_update(ssd);
+			ssd_border_update(ssd);
 			ssd_shadow_update(ssd);
 			/*
 			 * Not strictly necessary as ssd_titlebar_update()
@@ -257,23 +258,30 @@ ssd_update_geometry(struct ssd *ssd)
 			 */
 			ssd->state.was_maximized = maximized;
 		}
+		bool tiled_and_not_maximized = view_is_tiled(ssd->view) && !maximized;
+		if (ssd->state.was_tiled_not_maximized != tiled_and_not_maximized) {
+			ssd_titlebar_update(ssd);
+			ssd_border_update(ssd);
+			/* see above about being future proof */
+			ssd->state.was_tiled_not_maximized = tiled_and_not_maximized;
+		}
 		return;
 	}
 	ssd_extents_update(ssd);
-	ssd_border_update(ssd);
 	ssd_titlebar_update(ssd);
+	ssd_border_update(ssd);
 	ssd_shadow_update(ssd);
 	ssd->state.geometry = current;
 }
 
 void
-ssd_titlebar_hide(struct ssd *ssd)
+ssd_set_titlebar(struct ssd *ssd, bool enabled)
 {
-	if (!ssd || !ssd->titlebar.tree->node.enabled) {
+	if (!ssd || ssd->titlebar.tree->node.enabled == enabled) {
 		return;
 	}
-	wlr_scene_node_set_enabled(&ssd->titlebar.tree->node, false);
-	ssd->titlebar.height = 0;
+	wlr_scene_node_set_enabled(&ssd->titlebar.tree->node, enabled);
+	ssd->titlebar.height = enabled ? ssd->view->server->theme->title_height : 0;
 	ssd_border_update(ssd);
 	ssd_extents_update(ssd);
 	ssd_shadow_update(ssd);
@@ -309,7 +317,7 @@ ssd_destroy(struct ssd *ssd)
 bool
 ssd_part_contains(enum ssd_part_type whole, enum ssd_part_type candidate)
 {
-	if (whole == candidate) {
+	if (whole == candidate || whole == LAB_SSD_ALL) {
 		return true;
 	}
 	if (whole == LAB_SSD_PART_TITLEBAR) {
@@ -344,6 +352,21 @@ ssd_part_contains(enum ssd_part_type whole, enum ssd_part_type candidate)
 	return false;
 }
 
+enum ssd_mode
+ssd_mode_parse(const char *mode)
+{
+	if (!mode) {
+		return LAB_SSD_MODE_FULL;
+	}
+	if (!strcasecmp(mode, "none")) {
+		return LAB_SSD_MODE_NONE;
+	} else if (!strcasecmp(mode, "border")) {
+		return LAB_SSD_MODE_BORDER;
+	} else {
+		return LAB_SSD_MODE_FULL;
+	}
+}
+
 void
 ssd_set_active(struct ssd *ssd, bool active)
 {
